/*  BOUNCER.C  --  Screen saver that bounces a DIB.
 *
 *  (C) Copyright Microsoft Corp. 1991.  All rights reserved.
 *
 *  You have a royalty-free right to use, modify, reproduce and 
 *  distribute the Sample Files (and/or any modified version) in 
 *  any way you find useful, provided that you agree that 
 *  Microsoft has no warranty obligations or liability for any 
 *  Sample Application Files which are modified. 
 */

#include <windows.h> 
#include <mmsystem.h>
#include <scrnsave.h>

#include "bouncer.h"                            // BOUNCER

extern HANDLE hMainInstance;                    // instance handle
extern HANDLE hMainWindow;                      // main window handle

char szAppName[] = "ScreenSaver.Bouncer";       // Substitute your own name

char szDIBName[] = "BounceDIB";

char szIniName[] = "CONTROL.INI";
char szSpeedName[] = "Speed";
char szXPosName[] = "xPosition";
char szYPosName[] = "yPosition";
char szXVelocityName[] = "xVelocity";
char szGravityName[] = "Gravity";
char szSoundName[] = "Sound";
char szDIBNumName[] = "DIBNum";
char szPauseName[]= "Pause at bottom";

static HBITMAP hbmImage;        // image handle
static WORD wElapse;            // speed parameter
static WORD wTimer;             // timer id
static BOOL bBottom;            // TRUE if frog is at bottom of screen

static int xPos;                // current x position
static int yPos;                // current y position

static int xPosInit;            // initial x position
static int yPosInit;            // initial y position
static int xVelocInit;          // x initial velocity
static int nGravity;            // acceleration factor

static BOOL bSound;             // sound on/off flag
static BOOL bPause;             // stick at bottom of screen?
static HANDLE hresWave;         // handle to sound resource
static LPSTR lpWave;            // pointer to wave resource

static void MoveImage(HWND hWnd);
static void GetIniSettings(void);
static void WriteProfileInt(LPSTR key, LPSTR tag, int i);

/* End BOUNCER declarations
 */


/* ScreenSaverProc -- Main entry point for screen saver messages.
 *
 * Parameters -- Standard window message handler parameters.
 *
 *  Note that all messages go to the DefScreenSaverProc, except
 *  for ones we process.
 */
LONG FAR PASCAL ScreenSaverProc(HWND hWnd, WORD msg, WORD wParam, LONG lParam)
{
    RECT rc;
    static WORD wBottomCount;
    switch (msg)
    {
        case WM_CREATE:                             // BOUNCER
        {
            HANDLE hResInfo;

            /* Load the initial bounce settings.
             */
            GetIniSettings();

            /* Load the DIB image we want to use.
             */
            hbmImage = LoadBitmap(hMainInstance, szDIBName);

            /* Load and lock the sound resource
             */
            if( hResInfo = FindResource(hMainInstance, "Sound", "WAVE") )
            {
                if( hresWave = LoadResource(hMainInstance, hResInfo) )
                {
                    lpWave = LockResource(hresWave);
                }
            }

            /* Create a timer to move the image
             */
            wTimer = SetTimer(hWnd, ID_TIMER, wElapse, NULL);

            xPos = xPosInit;
            yPos = yPosInit;

            break;
        }

        case WM_TIMER:                              // BOUNCER
            if(bPause && bBottom)
            {
                if(++wBottomCount == 10)
                {
                    wBottomCount = 0;
                    bBottom = FALSE;
                }
                break;
            }

            /* Move the image around a bit
             */
            MoveImage(hWnd);

            break;

        case WM_DESTROY:                            // BOUNCER

            /* Destroy any objects we created
             */
            if( hbmImage ) DeleteObject(hbmImage);
            if( wTimer )   KillTimer(hWnd, ID_TIMER);
            if( lpWave )   UnlockResource(hresWave);
            if( hresWave ) FreeResource(hresWave);

            break;

        case WM_ERASEBKGND:
            GetClientRect(hWnd,&rc);
            FillRect(wParam,&rc,GetStockObject(BLACK_BRUSH));
            return 0L;

        default:
            break;
        }

    return DefScreenSaverProc(hWnd, msg, wParam, lParam);
}

/* RegisterDialogClasses -- Entry point for registering window
 * classes required by configuration dialog box.
 *
 * Params:  hWnd -- Handle to window
 *
 * Return:  None
 */
BOOL RegisterDialogClasses(HANDLE hInst)
{
    return TRUE;
}


/* ScreenSaverConfigureDialog -- Dialog box function for configuration
 * dialog.
 *
 * Params:  hWnd -- Handle to window
 *
 * Return:  None
 */
BOOL FAR PASCAL ScreenSaverConfigureDialog(HWND hDlg, WORD msg, WORD wParam, LONG lParam)
{
    switch (msg)
    {
        case WM_INITDIALOG:                         // BOUNCER
            GetIniSettings();
            SetDlgItemInt(hDlg, ID_SPEED,    wElapse,    FALSE);
            SetDlgItemInt(hDlg, ID_XPOS,     xPosInit,   TRUE);
            SetDlgItemInt(hDlg, ID_YPOS,     yPosInit,   TRUE);
            SetDlgItemInt(hDlg, ID_VELOCITY, xVelocInit, TRUE);
            SetDlgItemInt(hDlg, ID_GRAVITY,  nGravity,   TRUE);
            SendDlgItemMessage(hDlg, ID_SOUND, BM_SETCHECK, bSound, NULL);
            SendDlgItemMessage(hDlg, ID_PAUSE, BM_SETCHECK, bPause, NULL);

            return TRUE;

        case WM_COMMAND:                            // BOUNCER
            switch (wParam)
            {
                case IDOK:
                    wElapse    = GetDlgItemInt(hDlg, ID_SPEED,    NULL, FALSE);
                    xPosInit   = GetDlgItemInt(hDlg, ID_XPOS,     NULL, TRUE);
                    yPosInit   = GetDlgItemInt(hDlg, ID_YPOS,     NULL, TRUE);
                    xVelocInit = GetDlgItemInt(hDlg, ID_VELOCITY, NULL, TRUE);
                    nGravity   = GetDlgItemInt(hDlg, ID_GRAVITY,  NULL, TRUE);
                    bSound     = IsDlgButtonChecked(hDlg, ID_SOUND);
                    bPause     = IsDlgButtonChecked(hDlg, ID_PAUSE);

                    WriteProfileInt(szAppName, szSpeedName, wElapse);
                    WriteProfileInt(szAppName, szXPosName, xPosInit);
                    WriteProfileInt(szAppName, szYPosName, yPosInit);
                    WriteProfileInt(szAppName, szXVelocityName, xVelocInit);
                    WriteProfileInt(szAppName, szGravityName, nGravity);
                    WriteProfileInt(szAppName, szSoundName, bSound);
                    WriteProfileInt(szAppName, szPauseName, bPause);

                    EndDialog(hDlg, TRUE);
                    return TRUE;

                case IDCANCEL:
                    EndDialog(hDlg, FALSE);
                    return TRUE;

                default:
                    break;
            }

        default:
            break;
    }
    return FALSE;
}


/* THE REST OF THIS FILE IS SPECIFIC TO BOUNCER.
 *
 * Replace it with your own screen saver code.
 */


/* GetIniSettings -- Get initial bounce settings from WIN.INI
 *
 * Params:  hWnd -- Handle to window
 *
 * Return:  None
 */
static void GetIniSettings()
{
    wElapse =    GetPrivateProfileInt(szAppName, szSpeedName, DEF_SPEED, szIniName);
    xPosInit =   GetPrivateProfileInt(szAppName, szXPosName, DEF_INIT_XPOS, szIniName);
    yPosInit =   GetPrivateProfileInt(szAppName, szYPosName, DEF_INIT_YPOS, szIniName);
    xVelocInit = GetPrivateProfileInt(szAppName, szXVelocityName, DEF_INIT_VELOC, szIniName);
    nGravity =   GetPrivateProfileInt(szAppName, szGravityName, DEF_INIT_GRAVITY, szIniName);
    bSound =     GetPrivateProfileInt(szAppName, szSoundName, DEF_SOUND, szIniName);
    bPause =     GetPrivateProfileInt(szAppName, szPauseName, DEF_PAUSE, szIniName);
}

/* MoveImage -- Move image around the screen
 *
 * Params:  hWnd -- Handle to window
 *
 * Return:  None
 */
static void MoveImage(HWND hWnd) 
{
//    static int xPos = 10000;            // Current x value (force a reset)
//    static int yPos = 0;                // Current y value
    static int yVeloc = 0;              // Current y velocity

    int xPosOld, yPosOld;

    HDC hDC;                            // Handle to our window DC
    HDC hMemDC;                         // Handle to a memory DC
    BITMAP bm;                          // Bitmap info
    HBITMAP hbmOld;
    RECT rcWnd, rcFill;

    /* Get window size
     */
    GetClientRect(hWnd, &rcWnd);

    /* Get bitmap size
     */
    GetObject(hbmImage, sizeof(bm), (LPSTR)&bm);

    /* Update x- and y-position values
     */
    xPosOld = xPos;
    yPosOld = yPos;

    yVeloc += nGravity;

    xPos += xVelocInit;
    yPos += yVeloc;

    /* If we're at the bottom, BOUNCE!
     */
    if((yPos + bm.bmHeight > rcWnd.bottom) && (yVeloc > 0))
    {
        yVeloc = -yVeloc;                   // Reverse directions

        if (bSound && lpWave)               // Boing!!! 
        {                
            // do the multimedia bit
            sndPlaySound(lpWave, SND_ASYNC | SND_MEMORY | SND_NODEFAULT);
        }
        bBottom = TRUE;
    }

    /* If we're off the right of the screen, or off the bottom of the
     * screen, start over at beginning position and zero y-velocity.
     */
    if((xPos > rcWnd.right) || (yPos > rcWnd.bottom))
    {
        yVeloc = 0;
        xPos   = xPosInit;
        yPos   = yPosInit;
    }

    /* Get a DC to our window.  Create a compatible memory
     * DC and select our image bitmap into it so we can blit
     * it to the main window DC
     */
    hDC = GetDC(hWnd);
    hMemDC = CreateCompatibleDC(hDC);
    hbmOld = SelectObject(hMemDC, hbmImage);

    if(hbmOld)
    {
        /* Blit the image in the new position
         */
        BitBlt( hDC,                        // dest DC
                xPos,yPos,                  // dest origin
                bm.bmWidth,bm.bmHeight,     // dest extents
                hMemDC,                     // src DC
                0,0,                        // src origin
                SRCCOPY );                  // ROP code

        SelectObject(hMemDC, hbmOld);

        /* Tidy up where the old image was by filling exposed bits with
         * the background color
         *
         * This code assumes the image always moves to the right
         */
        rcFill.left   = xPosOld;                        // Left bits
        rcFill.top    = yPosOld;
        rcFill.right  = xPos;
        rcFill.bottom = rcFill.top + bm.bmHeight;

        FillRect(hDC, &rcFill, GetStockObject(BLACK_BRUSH));

        rcFill.right = rcFill.left + bm.bmWidth;        // Top or bottom bits
        if( yPos > yPosOld )
        {
            rcFill.bottom = yPos;                       // Top bits
        }
        else
        {
            rcFill.top = yPos + bm.bmHeight;            // Bottom bits
            rcFill.bottom = yPosOld + bm.bmHeight;
        }
        FillRect(hDC, &rcFill, GetStockObject(BLACK_BRUSH));
    }
    DeleteDC(hMemDC);
    ReleaseDC(hWnd, hDC);
}




/* WriteProfileInt - Write an unsigned integer value to CONTROL.INI.
 *
 * Params:  name - hMainWindow - Handle to main application window.
 *
 * Return:  None
 */
static void WriteProfileInt(LPSTR szAppName, LPSTR szKey, int i) 
{
    char achBuf[40];

    /* write out as unsigned because GetPrivateProfileInt() can't
     * cope with signed values!
     */
    wsprintf(achBuf, "%u", i);
    WritePrivateProfileString(szAppName,szKey,achBuf,szIniName);
}
